// A basic OpenGL program drawing various parametric curves
// (c) Burkhard Wuensche, 25th May 2009

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include <iostream>
using namespace std;
#include "Geometry.h"

const int windowWidth=600;
const int windowHeight=400;

double HermiteBasis1(double t){ return t*t*(2*t-3)+1;}		// B1(t)
double HermiteBasis2(double t){ return (-2*t+3)*t*t;}		// B2(t)
double HermiteBasis3(double t){ return t*(t*(t-2)+1);}		// B3(t)
double HermiteBasis4(double t){ return t*t*(t-1);}			// B4(t)

double BezierBasis1(double t){ t=1-t; return t*t*t;}		// B1(t)
double BezierBasis2(double t){ return 3*t*(1-t)*(1-t);}		// B2(t)
double BezierBasis3(double t){ return 3*t*t*(1-t);}			// B3(t)
double BezierBasis4(double t){ return t*t*t;}				// B4(t)

void display(void)
{
	// clear all pixels in frame buffer
	glClear(GL_COLOR_BUFFER_BIT);

	// Geometric constraints for the Hermite curve
	CVec2df p1(50.0, 50.0); 
	CVec2df p4(200.0 , 300.0);
	CVec2df r1(0.0 ,200.0);
	CVec2df r4(100.0, -200.0);
	CVec2df p;
	const int numPoints=20;
	double t;

	// Geometric constraints for the Bezier curve
	CVec2df q1=p1; q1[X]+=300; 
	CVec2df q4=p4; q4[X]+=300;  
	CVec2df q2=q1+r1/3;
	CVec2df q3=q4-r4/3;

	// Test properties of Hermite basis functions
	cout << "\n Properties of Hermite basis functions";
	cout << "\n B1(0)=" << HermiteBasis1(0.0) << "\t B1(1)=" << HermiteBasis1(1.0);
	cout << "\n B2(0)=" << HermiteBasis2(0.0) << "\t B2(1)=" << HermiteBasis2(1.0);
	cout << "\n B3(0)=" << HermiteBasis3(0.0) << "\t B3(1)=" << HermiteBasis3(1.0);
	cout << "\n B4(0)=" << HermiteBasis4(0.0) << "\t B4(1)=" << HermiteBasis4(1.0);

	// Draw end points of Hermite curve
	glPointSize(5.0f);
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_POINTS);
	glVertex2fv(p1.getArray());
	glVertex2fv(p4.getArray());
	glEnd();

	// Draw end tangents of Hermite curve
	glLineWidth(2.0f);
	glColor3f(0.0, 0.0, 1.0);
	glBegin(GL_LINES);
	glVertex2fv(p1.getArray());
	glVertex2fv((p1+r1).getArray());
	glVertex2fv(p4.getArray());
	glVertex2fv((p4+r4).getArray());
	glEnd();
	
	// Draw Hermite curve
	glLineWidth(2.0f);
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_LINE_STRIP);
	for(int i=0; i<numPoints; i++)
	{
		t=(double) i/(double) (numPoints-1);
		p=HermiteBasis1(t)*p1+HermiteBasis2(t)*p4+
		  HermiteBasis3(t)*r1+HermiteBasis4(t)*r4;
//		cout << "\n p(" << t << ")=\t" << p;
		glVertex2fv(p.getArray());
	}
	glEnd();


	// Test properties of Bezier basis functions
	cout << "\n Properties of Bezier basis functions";
	cout << "\n B1(0)=" << BezierBasis1(0.0) << "\t B1(1)=" << BezierBasis1(1.0);
	cout << "\n B2(0)=" << BezierBasis2(0.0) << "\t B2(1)=" << BezierBasis2(1.0);
	cout << "\n B3(0)=" << BezierBasis3(0.0) << "\t B3(1)=" << BezierBasis3(1.0);
	cout << "\n B4(0)=" << BezierBasis4(0.0) << "\t B4(1)=" << BezierBasis4(1.0);

	// Draw convex hull of Bezier control points
	glLineWidth(2.0f);
	glColor3f(0.0, 0.0, 1.0);
	glBegin(GL_LINE_LOOP);
	glVertex2fv(q1.getArray());
	glVertex2fv(q2.getArray());
	glVertex2fv(q3.getArray());
	glVertex2fv(q4.getArray());
	glEnd();

	// Draw control points of Bezier curve
	glPointSize(5.0f);
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_POINTS);
	glVertex2fv(q1.getArray());
	glVertex2fv(q2.getArray());
	glVertex2fv(q3.getArray());
	glVertex2fv(q4.getArray());
	glEnd();

	// Draw Bezier curve
	glLineWidth(2.0f);
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_LINE_STRIP);
	for(int i=0; i<numPoints; i++)
	{
		t=(double) i/(double) (numPoints-1);
		p=BezierBasis1(t)*q1+BezierBasis2(t)*q2+
		  BezierBasis3(t)*q3+BezierBasis4(t)*q4;
//		cout << "\n p(" << t << ")=\t" << p;
		glVertex2fv(p.getArray());
	}
	glEnd();

	// start processing buffered OpenGL routines 
	glFlush ();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1.0, 1.0, 1.0, 0.0);	// RGB-value for white

	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0,windowWidth,0,windowHeight);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


// create a single buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Hermite and Bezier Curve");
	init ();						// initialise view
	glutDisplayFunc(display);		// draw scene
	glutMainLoop();
	return 0;
}
